<?php
/**
 * Holler Box Pro Functions
 * @since       0.1.0
 */


// Exit if accessed directly
if( !defined( 'ABSPATH' ) ) exit;

if( !class_exists( 'Holler_Sales_Functions' ) ) {

    /**
     * Holler_Sales_Functions class
     *
     * @since       0.2.0
     */
    class Holler_Sales_Functions {

        /**
         * @var         Holler_Sales_Functions $instance The one true Holler_Sales_Functions
         * @since       0.2.0
         */
        private static $instance;
        public static $errorpath = '../php-error-log.php';
        public static $active = array();
        // sample: error_log("meta: " . $meta . "\r\n",3,self::$errorpath);

        /**
         * Get active instance
         *
         * @access      public
         * @since       0.2.0
         * @return      object self::$instance The one true Holler_Sales_Functions
         */
        public static function instance() {
            if( !self::$instance ) {
                self::$instance = new Holler_Sales_Functions();
                self::$instance->hooks();
            }

            return self::$instance;
        }


        /**
         * Include necessary files
         *
         * @access      private
         * @since       0.2.0
         * @return      void
         */
        private function hooks() {

            add_filter( 'hwp_custom_settings_save', array( $this, 'maybe_hide_avatar' ) );

            add_action( 'admin_init', array( $this, 'automatic_updates') );

            add_action( 'admin_init', array( $this, 'maybe_activate_deactivate_license') );

            add_action( 'hwp_type_settings', array( $this, 'type_settings') );

            add_action( 'hwp_license_fields', array( $this, 'license_fields' ) );

            add_action( 'wp_ajax_nopriv_hwp_fomo_ajax', array( $this, 'fomo_ajax' ) );
            add_action( 'wp_ajax_hwp_fomo_ajax', array( $this, 'fomo_ajax' ) );

        }

        /**
         * Add fomo setting
         *
         * @access      public
         * @since       0.1
         */
        public function type_settings( $id ) {

            ?>

            <label class="hwp-radio-withimage">
                <span class="text">FOMO</span>
                <img src="<?php echo Holler_Box_Sales_URL . '/assets/img/fomo-icon.png'; ?>" class="hwp-radio-image" />
                <input type="radio" name="hwp_type" value="fomo" <?php checked( "fomo", get_post_meta( $id, 'hwp_type', 1 ) ); ?> />
            </label>

            <?php
        }

        /**
         * Get sale with ajax
         *
         * @since       0.2.0
         * @return      void
         */
        public function fomo_ajax() {

            $int = ( isset( $_GET['int'] ) ? $_GET['int'] : 1 );

            // detect ecomm plugin
            if( class_exists('Easy_Digital_Downloads') ) {

                $content = $this->edd_sale( $int );

            } elseif( class_exists('WooCommerce') ) {
                
                $content = $this->woocommerce_sale( $int );

            } elseif( class_exists('Give') ) {
                
                $content = $this->give_query( $int );

            }

            wp_send_json_success( $content );
    
        }

        /**
         * Hook into settings save. If someone creates a FOMO box and saves it, delete the avatar email so it doesn't show.
         *
         * @since       0.2.0
         * @return      void
         */
        public function maybe_hide_avatar( $id ) {

            // Check for type. If it's fomo, delete the avatar.
            $type = get_post_meta( $id, 'hwp_type', 1 );
            if( $type === 'fomo' )
                delete_post_meta( $id, 'avatar_email' );

        }

        /**
         * Add latest EDD sale to Hollerbox
         *
         * @since       0.1.0
         * @return      void
         */
        public function edd_sale( $int ) {

            // If past 10 posts, get a random one
            if( $int > 10 )
                $int = rand( 1, 10 );

            $edd_payment = get_post_type_object( 'edd_payment' );
            $args = array(
                'post_type' => 'edd_payment',
                'post_status' => array( 'publish' ),
                'posts_per_page' => 1,
                'offset' => $int
            );

            $return = self::edd_payment_loop( $args, $content );

            return $return;

        }

        /**
         * Loop through EDD payments to get a FOMO sale
         *
         * @since       0.3.0
         * @return      void
         */
        public function edd_payment_loop( $args, $content ) {

            // The Query
            $the_query = new WP_Query( $args );

            if ( $the_query->have_posts() ) {
                while ( $the_query->have_posts() ) {
                    $the_query->the_post();

                    $post_id = get_the_ID();
                    $download_meta = get_post_meta( $post_id, '_edd_payment_meta', 1 );
                    $download_id = $download_meta['downloads'][0]['id'];

                    $customer_email = $download_meta['user_info']['email'];

                    $download = edd_get_download( $download_id );
                    $content = get_avatar( $customer_email, 50 );

                    $content .= '<span class="fomo-name">' . $download_meta['user_info']['first_name'] . ' purchased</span>';
                    $content .= '<span class="fomo-product"><a href="' . get_permalink( $download_id ) . '">' . $download->post_title . '</a></span>';

                }
                wp_reset_postdata();
            }

            return apply_filters( 'hwp_fomo_content', $content, $post_id );
        }

        /**
         * Add latest Woo sale to Hollerbox
         *
         * @since       0.1.0
         * @return      void
         */
        public function woocommerce_sale( $int ) {

            // If past 10 posts, get a random one
            if( $int > 10 )
                $int = rand( 1, 10 );

            $args = array( 
                'status' => 'wc-completed',
                'type' => 'shop_order',
                'posts_per_page' => 1,
                'offset'   => $int
             );

            $order = wc_get_orders( $args )[0];
            $id = $order->get_id();

            $items = $order->get_items();

            foreach ( $items as $item ) {
                $name = $item['name'];
                $product_id = $item['product_id'];
            }

            $first_name = $order->get_billing_first_name();
            $email = $order->get_billing_email();

            $content = get_avatar( $email, 50 );
            $content .= '<span class="fomo-name">' . $first_name . ' purchased</span>';
            $content .= '<span class="fomo-product"><a href="' . get_permalink( $product_id ) . '">' . $name . '</a></span>';

            return apply_filters( 'hwp_fomo_content', $content, $product_id );

        }

        /**
         *  Show GiveWP Donation Information
         *
         *  This query outputs the latest 3 transactions
         *  with the name of the donor and the amount
         *  https://github.com/WordImpress/Give-Snippet-Library/blob/master/useful-queries/sample-transaction-queries.php
         */
        public function give_query( $int ) {

            // If past 10 posts, get a random one
            if( $int > 10 )
                $int = rand( 1, 10 );

            // Query Arguments
            $args3 = array(
                'post_type'      => 'give_payment',
                'posts_per_page' => 1,
                'offset' => $int
            );

            $loop3 = new WP_Query( $args3 );

            if ( $loop3->have_posts() ) :

                /** 
                 * Get donation info
                 */
                while ( $loop3->have_posts() ) : $loop3->the_post();

                    $id = get_the_ID();
                    $meta = get_post_meta( $id );
                    // Transaction have their own metadata; let's get it.
                    $paymentmeta = $meta['_give_payment_meta'];
                    // The metadata is serialized. Let's pull that apart.
                    $getmeta = maybe_unserialize( $paymentmeta[0] );
                    // Now that we've got it, we can define the name
                    $firstname = $getmeta['user_info']['first_name'];
                    $email  = $getmeta['user_info']['email'];
                    $total = intval( $meta['_give_payment_total'][0] );
                    $formtitle = get_the_title( $getmeta['form_id'] );

                    /*
                     * If avatar doesn't exist, get form featured image, not working yet
                     */
                    $avatar_url = get_avatar_url( $email, array( 'size' => 50 ) );
                    // $form_thumb = get_the_post_thumbnail_url( $getmeta['form_id'] );
                    // if( strpos( $avatar_url, 'd=mm' ) != false && $form_thumb ) {
                    //     $avatar = '<img src="' . get_the_post_thumbnail_url( $getmeta['form_id'] ) . '" class="avatar avatar-50 photo" height="50" width="50">';
                    // } else {
                    //     $avatar = '<img alt="" src="' . $avatar_url . '" class="avatar avatar-50 photo" height="50" width="50">';
                    // }

                    $content .= $avatar = '<img alt="" src="' . $avatar_url . '" class="avatar avatar-50 photo" height="50" width="50">';

                    $content .= '<div class="fomo-text">';
                    $content .= '<span class="fomo-name"><strong>' . $firstname . '</strong> donated</span>';
                    $content .= '<span class="fomo-product">'. number_format( $total ,2) . '</span>';
                    $content .=  ' to <span class="fomo-product-title">' . $formtitle . '</span>';
                    $content .= '<span class="fomo-subtext">Thank you!</span>';
                    $content .= '</div>';

                endwhile;
                wp_reset_postdata(); // end of Query 1

            endif;

            wp_reset_query();

            return apply_filters( 'hwp_fomo_content', $content, $id );
        }

        /**
         *  Determine if at least 10 posts exist, otherwise we can't use offset properly
         */
        public function count_posts( $type ) {

            $num = wp_count_posts( $type );

            return $num->publish;

        }

        /**
         * Initialize EDD Software Licensing
         *
         * @access      public
         * @since       0.2
         */
        public function automatic_updates() {

            // retrieve our license key from the DB
            $license_key = trim( get_option( 'hwp_fomo_license' ) );
            // setup the updater
            $edd_updater = new EDD_SL_Plugin_Updater( HWP_SALES_STORE_URL, Holler_Box_Sales_File, array( 
                    'version'   => Holler_Box_Sales_VER,       // current version number
                    'license'   => $license_key,    // license key (used get_option above to retrieve from DB)
                    'item_id'     => 156,    // id of this plugin
                    'author'    => 'Scott Bolinger',  // author of this plugin
                    'url'           => home_url(),
                    'beta'          => false
                )
            );

            // save our license settings
            self::edd_license_save();

        }

        /**
         * Add license key fields
         *
         * @access      public
         * @since       0.3
         */
        public function license_fields() {

            $license_key = get_option( 'hwp_fomo_license' );
            $license_status = get_option( 'hwp_fomo_license_status' );

            $class = '';

            if( $license_status !== false && $license_status == 'valid' ) 
                $class = 'hwp-license-active';

            ?>
            <tr valign="top">
                <th scope="row" valign="top">
                    <?php _e('FOMO license key'); ?>
                </th>
                <td>
                    <input id="hwp_fomo_license" name="hwp_fomo_license" type="text" class="regular-text <?php echo $class; ?>" value="<?php esc_attr_e( $license_key ); ?>" />
                    <?php if( $license_status !== false && $license_status == 'valid' ) echo '<br><small>License active</small>'; ?>
                </td>
            </tr>

            <?php if( false !== $license_key ) : ?>
                <tr valign="top">
                    <th scope="row" valign="top">
                        
                    </th>
                    <td>
                        <?php if( $license_status !== false && $license_status == 'valid' ) : ?>
                            <input type="submit" class="button-secondary" name="fomo_license_deactivate" value="<?php _e('Deactivate License'); ?>"/>
                        <?php else: ?>
                                <input type="submit" class="button-primary" name="fomo_license_activate" value="<?php _e('Activate License'); ?>"/>
                        <?php endif; ?>
                    </td>
                </tr>
            <?php 
            endif;
        }

        /**
         * Save license settings
         *
         * @access      public
         * @since       0.4
         */
        public function edd_license_save() {

            // run a quick security check
            // if ( !isset( $_POST['hollerbox_settings_nonce'] ) || !wp_verify_nonce( $_POST['hollerbox_settings_nonce'], basename( __FILE__ ) ) )
            // return;

            // don't run if we are deactivating licenses
            if( isset( $_POST['fomo_license_deactivate'] ) || isset( $_POST['pro_license_deactivate'] ) )
                return;

            /** Licensing **/
            $license_key = get_option( 'hwp_fomo_license' );
            $license_status = get_option('hwp_fomo_license_status');

            $postval = ( isset( $_POST['hwp_fomo_license'] ) ? $_POST['hwp_fomo_license'] : '' );

            if( $postval != '' ) {

                update_option( 'hwp_fomo_license', sanitize_text_field( $postval ) );

            } 

            if( isset( $_POST['hwp_fomo_license'] ) && $_POST['hwp_fomo_license'] === '' ) {
                // delete license stuff if field is blank
                delete_option( 'hwp_fomo_license' );
                delete_option( 'hwp_fomo_license_status' );

            }

        }

        /**
         * Maybe Deactivate license
         *
         * @access      public
         * @since       0.2
         */
        public function maybe_activate_deactivate_license() {

            // listen for our activate button to be clicked
            if( isset( $_POST['fomo_license_deactivate'] ) ) {

                $license_key = trim( get_option( 'hwp_fomo_license' ) );

                $res = Holler_Pro_Admin::deactivate_license( $license_key, 156 );

                // wp_die( $res );

                // $license_data->license will be either "deactivated" or "failed"
                if( $res == 'deactivated' ) {
                    delete_option( 'hwp_fomo_license_status' );
                    $redirect = admin_url( 'edit.php?post_type=hollerbox&page=hollerbox' );
                } else {
                    $base_url = admin_url( 'edit.php?post_type=hollerbox&page=hollerbox' );
                    $redirect = add_query_arg( array( 'sl_activation' => 'false', 'message' => urlencode( $res ) ), $base_url );
                }

                wp_redirect( $redirect );
            } else if( isset( $_POST['fomo_license_activate'] ) && isset( $_POST['hwp_fomo_license'] ) ) {

                $license_key = $_POST['hwp_fomo_license'];

                $res = Holler_Pro_Admin::activate_license( $license_key, 156 );

                // $res is any error messages from license activation. Display them to user on redirect if necessary.
                if ( ! empty( $res['msg'] ) ) {
                    $base_url = admin_url( 'edit.php?post_type=hollerbox&page=hollerbox' );
                    $redirect = add_query_arg( array( 'sl_activation' => 'false', 'message' => urlencode( $res['msg'] ) ), $base_url );
                    wp_redirect( $redirect );
                    exit();
                }

                // $license_data->license will be either "valid" or "invalid"
                update_option( 'hwp_fomo_license_status', $res['status'] );

                wp_redirect( admin_url( 'edit.php?post_type=hollerbox&page=hollerbox' ) );

            }
        }

    }

    $holler_sales_functions = new Holler_Sales_Functions();
    $holler_sales_functions->instance();

} // end class_exists check